@@ -26,11 +26,22 @@
 
 static GtkObjectClass *parent_class = NULL;
 
+static GHashTable     *cache = NULL;
+
 #define CLASS(o) BONOBO_CONFIG_ARCHIVER_CLASS (GTK_OBJECT(o)->klass)
 
 #define PARENT_TYPE BONOBO_CONFIG_DATABASE_TYPE
 #define FLUSH_INTERVAL 30 /* 30 seconds */
 
+/* Small Bonobo bug here... */
+
+#undef BONOBO_RET_EX
+#define BONOBO_RET_EX(ev)		\
+	G_STMT_START{			\
+		if (BONOBO_EX (ev))	\
+			return;		\
+	}G_STMT_END
+
 extern int daytime;
 
 static DirEntry *
@@ -211,12 +222,32 @@ real_get_value (BonoboConfigDatabase *db,
 	return value;
 }
 
+/* Produce a new, dynamically allocated string with the OAF IID of the listener
+ * associated with the given backend id
+ */
+
+static gchar *
+get_listener_oafiid (const gchar *backend_id) 
+{
+	gchar *oafiid, *tmp, *tmp1;
+
+	tmp = g_strdup (backend_id);
+	if ((tmp1 = strstr (tmp, "-properties")) != NULL) *tmp1 = '\0';
+	oafiid = g_strconcat ("OAFIID:Bonobo_Listener_Config_", tmp, NULL);
+	g_free (tmp);
+
+	return oafiid;
+}
+
 static void
 real_sync (BonoboConfigDatabase *db, 
 	   CORBA_Environment    *ev)
 {
 	BonoboConfigArchiver *archiver_db = BONOBO_CONFIG_ARCHIVER (db);
-	BonoboArg *arg;
+	BonoboArg            *arg;
+
+	gchar                *listener_oafiid;
+	Bonobo_Listener       listener;
 
 	if (!db->writeable)
 		return;
@@ -229,9 +260,27 @@ real_sync (BonoboConfigDatabase *db,
 	location_client_store_xml (archiver_db->location, archiver_db->backend_id, 
 				   archiver_db->doc, ConfigArchiver_STORE_MASK_PREVIOUS, ev);
 
+	BONOBO_RET_EX (ev);
+
+	/* Try to find a listener to apply the settings. If we can't, don't
+	 * worry about it
+	 */
+
+	listener_oafiid = get_listener_oafiid (archiver_db->backend_id);
+	listener = bonobo_get_object (listener_oafiid, "IDL:Bonobo/Listener:1.0", ev);
+	g_free (listener_oafiid);
+
+	if (BONOBO_EX (ev)) {
+		listener = CORBA_OBJECT_NIL;
+		CORBA_exception_init (ev);
+	}
+
 	arg = bonobo_arg_new (BONOBO_ARG_NULL);
 	bonobo_event_source_notify_listeners (archiver_db->es, "Bonobo/ConfigDatabase:sync", arg, ev);
 	bonobo_arg_release (arg);
+
+	if (listener != CORBA_OBJECT_NIL)
+		bonobo_object_release_unref (listener, NULL);
 }
 
 static void
@@ -500,16 +549,13 @@ bonobo_config_archiver_destroy (GtkObject *object)
 
 	DEBUG_MSG ("Enter");
 
+	gtk_object_ref (object);
+
 	CORBA_exception_init (&ev);
 
 	if (archiver_db->moniker != NULL) {
-		bonobo_url_unregister ("BONOBO_CONF:ARCHIVER", archiver_db->moniker, &ev);
+		g_hash_table_remove (cache, archiver_db->moniker);
 		g_free (archiver_db->moniker);
-
-		if (BONOBO_EX (&ev)) {
-			g_critical ("Could not unregister the archiver URL");
-			CORBA_exception_init (&ev);
-		}
 	}
 
 	if (archiver_db->listener_id != 0) {
@@ -542,6 +588,8 @@ bonobo_config_archiver_destroy (GtkObject *object)
 	
 	parent_class->destroy (object);
 
+	gtk_object_unref (object);
+
 	DEBUG_MSG ("Exit");
 }
 
@@ -737,37 +785,35 @@ new_rollback_cb (BonoboListener        *listener,
 	bonobo_arg_release (arg);
 }
 
-Bonobo_ConfigDatabase
+BonoboConfigArchiver *
 bonobo_config_archiver_new (Bonobo_Moniker               parent,
 			    const Bonobo_ResolveOptions *options,
 			    const char                  *moniker,
 			    CORBA_Environment           *ev)
 {
 	BonoboConfigArchiver    *archiver_db;
-	Bonobo_ConfigDatabase    db;
 	ConfigArchiver_Archive   archive;
 	ConfigArchiver_Location  location;
-	gchar                   *moniker_tmp;
 	gchar                   *backend_id;
 	gchar                   *location_id;
 	struct tm               *date;
 
 	g_return_val_if_fail (backend_id != NULL, NULL);
 
-	/* Check the Bonobo URL database to see if this archiver database has
-	 * already been created, and return it if it has */
+	/* Check whether the database object already exists, and return it if it does */
 
-	moniker_tmp = g_strdup (moniker);
-	db = bonobo_url_lookup ("BONOBO_CONF:ARCHIVER", moniker_tmp, ev);
-	g_free (moniker_tmp);
+	if (cache == NULL)
+		cache = g_hash_table_new (g_str_hash, g_str_equal);
 
-	if (BONOBO_EX (ev)) {
-		db = CORBA_OBJECT_NIL;
-		CORBA_exception_init (ev);
+	archiver_db = g_hash_table_lookup (cache, moniker);
+
+	if (archiver_db != NULL) {
+		g_message ("%s: Found database %s in the hash table", __FUNCTION__, moniker);
+		bonobo_object_ref (BONOBO_OBJECT (archiver_db));
+		return archiver_db;
 	}
 
-	if (db != CORBA_OBJECT_NIL)
-		return bonobo_object_dup_ref (db, NULL);
+	g_message ("%s: Did NOT find database %s in the hash table", __FUNCTION__, moniker);
 
 	/* Parse out the backend id, location id, and rollback date from the
 	 * moniker given */
@@ -826,8 +872,6 @@ bonobo_config_archiver_new (Bonobo_Moniker               parent,
 		g_free (filename);
 
 		if (archiver_db->doc == NULL) {
-			bonobo_object_release_unref (archiver_db->location, NULL);
-			bonobo_object_release_unref (archiver_db->archive, NULL);
 			bonobo_object_unref (BONOBO_OBJECT (archiver_db));
 			return CORBA_OBJECT_NIL;
 		}
@@ -883,13 +927,8 @@ bonobo_config_archiver_new (Bonobo_Moniker               parent,
 	else
 		archiver_db->listener_id = 0;
 
-	/* Prepare to return the database object */
+	g_message ("%s: Inserting database %s in the cache", __FUNCTION__, moniker);
+	g_hash_table_insert (cache, g_strdup (moniker), archiver_db);
 
-	db = CORBA_Object_duplicate (BONOBO_OBJREF (archiver_db), NULL);
-
-	moniker_tmp = g_strdup (moniker);
-	bonobo_url_register ("BONOBO_CONF:ARCHIVER", moniker_tmp, NULL, db, ev);
-	g_free (moniker_tmp);
-
-	return db;
+	return archiver_db;
 }
